package ru.dbotthepony.mc.otm.triggers

import com.google.gson.JsonObject
import com.google.gson.JsonParseException
import com.google.gson.JsonPrimitive
import com.mojang.serialization.Codec
import com.mojang.serialization.codecs.RecordCodecBuilder
import net.minecraft.advancements.Criterion
import net.minecraft.advancements.critereon.*
import net.minecraft.resources.ResourceLocation
import net.minecraft.server.level.ServerPlayer
import ru.dbotthepony.mc.otm.OverdriveThatMatters
import ru.dbotthepony.mc.otm.core.set
import ru.dbotthepony.mc.otm.core.toJsonStrict
import ru.dbotthepony.mc.otm.data.Codec2TriggerSerializer
import ru.dbotthepony.mc.otm.data.minRange
import java.util.Optional

object AndroidTravelUnderwater : SimpleCriterionTrigger<AndroidTravelUnderwater.Instance>() {
	val id = ResourceLocation(OverdriveThatMatters.MOD_ID, "android_walk_underwater")

	override fun createInstance(pJson: JsonObject, pPlayer: Optional<ContextAwarePredicate>, pContext: DeserializationContext): Instance {
		return codec.fromJson(pJson, pPlayer, pContext)
	}

	fun trigger(player: ServerPlayer, travelled: Double) {
		trigger(player) {
			it.distanceToTravel <= travelled
		}
	}

	val codec = Codec2TriggerSerializer<Instance> { p ->
		RecordCodecBuilder.create {
			it.group(
				Codec.DOUBLE.minRange(0.0).fieldOf("distanceToTravel").forGetter(Instance::distanceToTravel),
				p.playerPredicate.forGetter { it.playerPredicate() }
			).apply(it, ::Instance)
		}
	}

	class Instance(val distanceToTravel: Double, playerPredicate: Optional<ContextAwarePredicate> = Optional.empty()) : AbstractCriterionTriggerInstance(playerPredicate) {
		override fun serializeToJson(): JsonObject {
			return codec.toJson(this)
		}

		fun criterion() = Criterion(AndroidTravelUnderwater, this)
	}
}
